<?php

namespace Clonable\Translator\Model\Product;

use Exception;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\App\CacheInterface;
use Magento\Framework\MessageQueue\QueueInterface;
use Magento\Framework\Serialize\Serializer\Json;
use Magento\Store\Model\StoreManagerInterface;
use Magento\Catalog\Api\Data\ProductInterface;
use Magento\Catalog\Api\ProductRepositoryInterface;
use Magento\Catalog\Model\ResourceModel\Product\Action;
use Magento\CatalogUrlRewrite\Model\ProductUrlRewriteGenerator;
use Magento\UrlRewrite\Model\Exception\UrlAlreadyExistsException;
use Magento\UrlRewrite\Model\UrlFinderInterface;
use Magento\UrlRewrite\Model\UrlPersistInterface;
use Clonable\Translator\Model\AbstractTranslator;
use Clonable\Translator\Api\Data\Product\TranslatorProductInterface;
use Clonable\Translator\Model\ConfigManager;
use Clonable\Translator\Api\Service\ClonableTranslatorApiInterface;
use Clonable\Translator\Api\ReportLogsRepositoryInterface;
use Clonable\Translator\Model\ReportLogsFactory;
use Clonable\Translator\Model\ReportLogs;
use Clonable\Translator\Model\Logger\Logger;



class TranslatorProduct extends AbstractTranslator implements TranslatorProductInterface
{
    /**
     * @var ProductRepositoryInterface
     */
    private ProductRepositoryInterface $productRepository;


    /**
     * @var ProductUrlRewriteGenerator
     */
    private ProductUrlRewriteGenerator $productUrlRewriteGenerator;

    /**
     * @var Action
     */
    private Action $action;

    /**
     * @var Logger
     */
    private Logger $logger;

    /**
     * @var UrlPersistInterface
     */
    protected UrlPersistInterface $urlPersist;

    /**
     * @var StoreManagerInterface
     */
    protected StoreManagerInterface $storeManager;

    /**
     * @var ConfigManager
     */
    protected ConfigManager $configManager;

    /**
    * @var ClonableTranslatorApiInterface
    */
    private ClonableTranslatorApiInterface $clonableTranslatorAPIInterface;

    private ReportLogsRepositoryInterface $reportLogsRepository;

    private ReportLogsFactory $reportLogsFactory;

    public function __construct(
        UrlPersistInterface $urlPersist,
        UrlFinderInterface $urlFinder,
        CacheInterface $cache,
        ProductRepositoryInterface $productRepository,
        ProductUrlRewriteGenerator $productUrlRewriteGenerator,
        Action $action,
        Json $json,
        Logger $logger,
        StoreManagerInterface $storeManager,
        ConfigManager $configManager,
        ClonableTranslatorApiInterface $clonableTranslatorAPIInterface,
        ReportLogsRepositoryInterface $reportLogsRepository,
        ReportLogsFactory $reportLogsFactory
    ){
        parent::__construct($urlPersist, $urlFinder, $cache, $json, $storeManager, $configManager, $clonableTranslatorAPIInterface);
        $this->productRepository = $productRepository;
        $this->productUrlRewriteGenerator = $productUrlRewriteGenerator;
        $this->action = $action;
        $this->logger = $logger;
        $this->reportLogsRepository = $reportLogsRepository;
        $this->reportLogsFactory = $reportLogsFactory;
    }

    /**
     * @param QueueInterface $queue
     * @return void
     */
    public function processTranslationQueue(QueueInterface $queue): void
    {
        $storeIds = $this->getStoresForTranslate();

        while ($message = $queue->dequeue()) {
            try {
                $productSku = $this->json->unserialize($message->getBody(), true);
                if (empty($storeIds)) {
                    $this->logger->warning('No store IDs available for translation product SKU ' . $productSku);
                    $this->reportLogsRepository->save($this->createReportLog('No store IDs available for translation product SKU ' . $productSku));
                    $queue->reject($message);
                    continue;
                }
                foreach ($storeIds as $storeId) {
                    $startScriptTime = microtime(true);
                    $product = $this->getProductBySku($productSku, $storeId);
                    $updateAttrTime = $this->updateProductAttributes($product);
                    $this->updateUrlRewrite($this->getProductBySku($productSku, $storeId));
                    $this->updateExcludeClonableAutoTranslation($product->getId(), true, $product->getStoreId());
                }
                $queue->acknowledge($message);
                $this->logger->info("Product SKU " . $productSku . "  updated successfully.");
                $endScriptTime = microtime(true);
                $totalTime = microtime(true) - $startScriptTime;
                $translateTime = $updateAttrTime['end_translate_time'];
                $updateProduct = $totalTime - $translateTime;
                $this->logger->info(
                    "Time taken for updating product SKU {$productSku}: {$updateProduct} seconds (Translate: {$translateTime} seconds), Store ID: {$storeId}"
                );
            } catch (Exception $e) {
                $this->reportLogsRepository->save($this->createReportLog('Error updating product with SKU ' . ($productSku ?? 'N/A') . ': ' . $e->getMessage(), $storeId));
                $this->logger->error('Error updating product with SKU ' . ($productSku ?? 'N/A') . ': ' . $e->getMessage());
                $queue->reject($message, true, $e->getMessage());
            }
        }
    }

    /**
     * @param $product
     * @throws Exception
     */
    private function updateProductAttributes($product)
    {
        try {
            $startTranslateTime = microtime(true);
            $updatedAttributes  = ['name' => $this->translateText($product->getName(), $product->getStore())];
            if($this->configManager->isEnabledTranslateUrlKey($product->getStoreId())) {
                $updatedAttributes ['url_key'] = $this->translateUrlKey($product->getUrlKey(), $product->getStore());
            }
            $endTranslateTime = microtime(true);
            $startUpdateAttrTime = microtime(true);
            $this->action->updateAttributes([$product->getId()], $updatedAttributes, $product->getStoreId());
            $endUpdateAttrTime = microtime(true);
        } catch (Exception $e) {
            throw new Exception($e->getMessage());
        }
        return [
            'end_translate_time' => $endTranslateTime - $startTranslateTime,
            'end_update_attr_time' => $endUpdateAttrTime - $startUpdateAttrTime
        ];
    }

    /**
     * @param $product
     * @throws Exception
     */
    private function updateUrlRewrite($product)
    {
        if ($this->configManager->isEnabledTranslateUrlKey($product->getStoreId())) {
            $startUrlRewriteTime = microtime(true);
            $product->setData('save_rewrites_history', true);
            $urls = $this->productUrlRewriteGenerator->generate($product);
            try {
                $this->urlPersist->replace($urls);
            } catch (UrlAlreadyExistsException $e) {
                $this->updateAlreadyExistsUrl($product);
            } catch (Exception $e) {
                throw new Exception($e->getMessage());
            }
            $endUrlRewriteTime = microtime(true);
            return $endUrlRewriteTime - $startUrlRewriteTime;
        }
        return null;
    }

    /**
     * @throws Exception
     */
    private function updateAlreadyExistsUrl($product): void
    {
        $this->action->updateAttributes([$product->getId()], ['url_key' => $product->getUrlKey() . '-' . $product->getId()], $product->getStoreId());
        $this->updateUrlRewrite($this->getProductBySku($product->getSku()));
    }

    /**
     * @param $sku
     * @param null $storeId
     * @return ProductInterface
     * @throws NoSuchEntityException
     */
    private function getProductBySku($sku, $storeId = null): ProductInterface
    {
        return $this->productRepository->get($sku, true, $storeId, true);
    }

    /**
     * @throws Exception
     */
    public function updateExcludeClonableAutoTranslation($productId, $exclude, $storeId): void
    {

        try {
            $this->action->updateAttributes(
                [$productId],
                ['exclude_clonable_autotranslation' => $exclude],
                $storeId
            );
        } catch (Exception $e) {
            throw new Exception($e->getMessage());
        }

    }

    /**
     *
     * @param string $message
     * @param string $storeId
     * @return ReportLogs
     */
    public function createReportLog(string $message, string $storeId = '0'): ReportLogs
    {
        $reportLog = $this->reportLogsFactory->create();
        $reportLog->setMessage($message);
        $reportLog->setStoreId($storeId);
        return $reportLog;
    }
}
