<?php

namespace Clonable\Translator\Model\Service;

use Exception;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\HTTP\Client\Curl;
use Magento\Store\Model\Store;
use Clonable\Translator\Api\Service\ClonableTranslatorApiInterface;
use Clonable\Translator\Model\ConfigManager;
use Clonable\Translator\Model\Logger\Logger;

class ClonableTranslatorApi implements ClonableTranslatorApiInterface
{
    /**
     * @var Curl
     */
    private Curl $curl;

    /**
     * @param ConfigManager $configManager
     */
    private ConfigManager $configManager;

    private Logger $logger;

    public function __construct(
        Curl $curl,
        ConfigManager $configManager,
        Logger $logger
    ){
        $this->curl = $curl;
        $this->configManager = $configManager;
        $this->logger = $logger;
    }

    /**
     * Translate a URL using the external API.
     *
     * @param string $url
     * @param Store $store
     * @return string
     * @throws LocalizedException
     */
    public function translateUrlKey(string $url, Store $store): string
    {
        $maxAttempts = 5;
        $initialDelay = 1;
        $data = $this->prepareUrlTranslationData($url, $store);
        $headers = $this->prepareHeaders($store);

        $this->logger->info('Request Body: ' . json_encode($data));

        for ($attempt = 0; $attempt < $maxAttempts; $attempt++) {
                $this->curl->setHeaders($headers);
                $this->curl->post(ConfigManager::API_URL_TRANSLATE, json_encode($data));

                if ($this->curl->getStatus() === 200) {
                    $response = $this->curl->getBody();
                    return basename(parse_url(json_decode($response)->translation, PHP_URL_PATH));
                } else {
                    sleep($initialDelay * pow(2, $attempt));
                }
        }

        throw new LocalizedException(__('Failed to translate URL after multiple attempts.'));
    }

    /**
     * Translate a product name using the external API.
     *
     * @param string $text
     * @param Store $store
     * @return string
     * @throws LocalizedException
     */
    public function translateText(string $text, Store $store): string
    {

        $maxAttempts = 5;
        $initialDelay = 1;
        $data = $this->prepareTextTranslationData($text, $store);
        $headers = $this->prepareHeaders($store);

        for ($attempt = 0; $attempt < $maxAttempts; $attempt++) {
                $this->curl->setHeaders($headers);
                $this->curl->post(ConfigManager::API_URL_TEXT, json_encode($data));

                if ($this->curl->getStatus() === 200) {
                    $response = $this->curl->getBody();
                    return json_decode($response)->translation;
                } else {

                sleep($initialDelay * pow(2, $attempt));
            }
        }

        throw new LocalizedException(__('Failed to translate text after multiple attempts.'));
    }

    /**
     * Prepare data for URL translation.
     *
     * @param string $url
     * @param Store $store
     * @return array
     */
    private function prepareUrlTranslationData(string $url, Store $store): array
    {
        return [
            'url' => $store->getBaseUrl() . $store->getCode() . '/' . $url,
            'clone_id' => $this->configManager->getCloneId($store->getStoreId()),
            'site_id' => $this->configManager->getSiteId($store->getStoreId()),
            'original_language' => $this->configManager->getOriginalLanguage($store->getStoreId()),
            'target_language' => $this->configManager->getTargetLanguage($store->getStoreId()),
            'create_new' => true
        ];
    }

    /**
     * Prepare data for product name translation.
     *
     * @param string $name
     * @param Store $store
     * @return array
     */
    private function prepareTextTranslationData(string $name, Store $store): array
    {
        return [
            "content" => $name,
            "text_only" => true,
            "editor_mode" => false,
            "clone_id" => $this->configManager->getCloneId($store->getStoreId()),
        ];
    }

    /**
     * Prepare headers for API requests.
     *
     * @param Store $store
     * @return array
     */
    private function prepareHeaders(Store $store): array
    {
        return [
            'Authorization' => 'Bearer ' . $this->configManager->decryptApiKey($this->configManager->getApiKey($store->getStoreId())),
            'Accept' => 'application/json',
            'Content-Type' => 'application/json'
        ];
    }
}
